/*
  MPIO block device driver, version 0.0.1
  Copyright (C) 2002 Yury Umanets (umka@namesys.com)

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation, version 2.
*/

#ifndef MPIO_H
#define MPIO_H

#include <linux/usb.h>

struct usb_iface {
	struct usb_device *udev;            /* save off the usb device pointer */
	struct usb_interface *interface;    /* the interface for this device */

	/* Here also will be input and output buffers */
};

enum mpio_ctl {
	CTL_GET_VERSION = 0x01,
	CTL_GET_BLOCK   = 0x02,
	CTL_PUT_SECTOR  = 0x03,
	CTL_DEL_BLOCK   = 0x04,
	CTL_GET_SECTOR  = 0x06,
	CTL_GET_SPAREA  = 0x07,
	CTL_PUT_BLOCK   = 0x08
};

typedef enum mpio_ctl mpio_ctl_t;

enum mpio_stor {
	STR_INTERNAL    = 0x01,
	STR_EXTERNAL    = 0x10
};

typedef enum mpio_stor mpio_stor_t;

enum mpio_model {
	MPIO_DME        = 0x00,
	MPIO_DMG        = 0x01,
	MPIO_DMG_PLUS   = 0x02,
	MPIO_DMB        = 0x03,
	MPIO_DMB_PLUS   = 0x04,
	MPIO_DMK        = 0x05,
	MPIO_UNKNOWN    = 0x06,
	MPIO_LAST
};

typedef enum mpio_model mpio_model_t;

struct mpio_packet {
	unsigned char body[0x40];
};

struct smartcard {
	int size;
};

struct mpio {

	/* Device len in bytes */
	int size;

	/* Device model */
	mpio_model_t model;
	
	/* MPIO version info packet */
	struct mpio_packet version;

	/* Internal smartcard */
	struct smartcard internal;

	/* USB interface */
	struct usb_iface iface;

	/* Wait queue for sleeping on it */
	wait_queue_head_t wq;

	/* Flag for preventing more than one initializing */
	int initialized;

	/* Input and output buffers */
	void *ibuff, *obuff;

	/* Prefetch range */
	int pstart, pcount;
};

#define SECTOR_SIZE  (512)
#define SPARE_SIZE   (64)
#define TRANS_SIZE   (SECTOR_SIZE + SPARE_SIZE)

extern int mpio_init(struct mpio *mpio);
extern void mpio_fini(struct mpio *mpio);
extern void mpio_free(struct mpio *mpio);

extern struct mpio *mpio_create(struct usb_device *udev, int ifnum);

extern int mpio_read(struct mpio *mpio, void *buff, int sector);
extern int mpio_write(struct mpio *mpio, void *buff, int sector);

#endif
